/*
 * PHPify - Browserify transform
 * Copyright (c) Dan Phillimore (asmblah)
 * https://github.com/uniter/phpify
 *
 * Released under the MIT license
 * https://github.com/uniter/phpify/raw/master/MIT-LICENSE.txt
 */

'use strict';

var _ = require('microdash'),
    path = require('path'),
    Promise = require('lie');

/**
 * Virtual FileSystem for use in the browser with compiled PHP modules
 *
 * @constructor
 */
function FileSystem() {
    /**
     * A special function, generated by the PHP module compiler,
     * that can be called to fetch the module wrapper of any compiled modules
     * or return true/false to determine whether they exist
     *
     * @type {Function|null}
     */
    this.phpModuleFactoryFetcher = null;
}

_.extend(FileSystem.prototype, {
    /**
     * Fetches the module wrapper factory function for a compiled PHP module,
     * if it exists in the compiled bundle
     *
     * @param {string} filePath
     * @returns {Function}
     * @throws {Error} Throws when the specified compiled module does not exist
     */
    compilePHPFile: function (filePath) {
        var fileSystem = this,
            moduleFactory;

        filePath = fileSystem.realPath(filePath);
        moduleFactory = fileSystem.phpModuleFactoryFetcher(filePath, false);

        if (moduleFactory === null) {
            throw new Error('File "' + filePath + '" is not in the compiled PHP file map');
        }

        return moduleFactory;
    },

    /**
     * Initializes the FileSystem with a fetcher function that will return
     * the wrapper factory function for compiled PHP modules
     *
     * @param {Function} phpModuleFactoryFetcher
     */
    init: function (phpModuleFactoryFetcher) {
        this.phpModuleFactoryFetcher = phpModuleFactoryFetcher;
    },

    /**
     * Determines whether the specified directory path exists in the FileSystem.
     * Currently always returns true, as we cannot be sure from the info we have
     *
     * @returns {boolean}
     */
    isDirectory: function () {
        // TODO: Implement once we have support for non-PHP files in the VFS
        return false;
    },

    /**
     * Determines whether the specified file exists in the FileSystem.
     * Currently only compiled PHP modules can be in the FileSystem, so only those
     * may be detected.
     *
     * @param {string} filePath
     * @returns {boolean}
     */
    isFile: function (filePath) {
        var fileSystem = this;

        filePath = fileSystem.realPath(filePath);

        return fileSystem.phpModuleFactoryFetcher(filePath, true);
    },

    /**
     * Opens a Stream for the specified file asynchronously
     *
     * @param {string} filePath
     * @returns {Promise} Resolves with a Stream for the file on success, rejects on failure
     */
    open: function (filePath) {
        return new Promise(function (resolve, reject) {
            reject(new Error('Could not open "' + filePath + '" :: Streams are not currently supported by PHPify'));
        });
    },

    /**
     * Opens a Stream for the specified file synchronously
     *
     * @param {string} filePath
     * @returns {Stream}
     */
    openSync: function (filePath) {
        throw new Error('Could not open "' + filePath + '" :: Streams are not currently supported by PHPify');
    },

    /**
     * Converts the specified module path to a full one,
     * normalizing any parent- or current-directory symbols
     *
     * @param {string} filePath
     * @returns {string}
     */
    realPath: function (filePath) {
        filePath = path.normalize(filePath);

        return filePath;
    },

    /**
     * Deletes a file or folder asynchronously
     *
     * @param {string} filePath
     * @returns {Promise} Resolves on success, rejects on failure
     */
    unlink: function (filePath) {
        return new Promise(function (resolve, reject) {
            reject(new Error('Could not delete "' + filePath + '" :: not currently supported by PHPify'));
        });
    },

    /**
     * Deletes a file or folder synchronously
     *
     * @param {string} filePath
     */
    unlinkSync: function (filePath) {
        throw new Error('Could not delete "' + filePath + '" :: not currently supported by PHPify');
    }
});

module.exports = FileSystem;
